# from tests.output_betterproto.map import Enum, Test


def test_map():
    # msg = Test(counts={"a": 1, "b": 2})
    # assert msg == Test.parse(bytes(msg))
    # assert msg == Test.from_dict(msg.to_dict())

    pass
    # msg = Test(map_enum={1: Enum.ONE, 2: Enum.TWO})
    # assert msg == Test.parse(bytes(msg))
    # assert msg == Test.from_dict(msg.to_dict())
